package com.tillekesoft.tensorflowlite;

import android.graphics.Bitmap;

import org.tensorflow.lite.TensorFlowLite;

import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import anywheresoftware.b4a.BA;

@BA.Version(0.20f)
@BA.ShortName("TensorflowLite")
@BA.Author(value = "moster67/Mikael Osterhed")
@BA.DependsOn(values = {"tensorflow-lite-1.10.0.aar", "guava-26.0-android"})

public class TensorflowWrapper implements Classifier {
    private BA ba;
    private String eventName;
    private TensorFlowLite mTensorFlowLite = null;
    private String model_path = ""; //"mobilenet_quant_v1_224.tflite";
    private String lable_path = ""; //"labels.txt";
    private int input_size = 224; //default value = 224

    private Executor executor = Executors.newSingleThreadExecutor();
    private Classifier classifier;

    /**
     * Initializes TensorflowLite. Reads Model-file (tflite or lite) and label-file from Assets-folder.
     * Default value of Input-size is 224
     */
    public void Initialize(final BA ba, String EventName, int InputSize, String AssetModel, String AssetLabel) {
        this.eventName = EventName.toLowerCase(BA.cul);
        this.ba = ba;
        this.input_size = InputSize;
        this.model_path = AssetModel;
        this.lable_path = AssetLabel;
        initTensorFlowAndLoadModel();
    }

    private void initTensorFlowAndLoadModel() {
        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    classifier = TensorFlowImageClassifier.create(
                            ba.applicationContext.getAssets(),
                            model_path,
                            lable_path,
                            input_size);
                } catch (final Exception e) {
                    throw new RuntimeException("Error initializing TensorFlow!", e);
                }
            }
        });
    }



    @BA.Hide
    public void Destroy() {
        //Destroys/closes classifier. Should be run in the onPause-event.
        //seems to crash B4A-app. Needs to be verified.....disabled for the time being
        executor.execute(new Runnable() {
            @Override
            public void run() {
                classifier.close();
            }
        });
    }


    /**
     * Requests TensoflowLite/classifier to recognize image
     * and return possible results.
     */
    @Override
    public List<Recognition> recognizeImage(Bitmap bitmap) {
        return classifier.recognizeImage(bitmap);
    }


    @BA.Hide
    @Override
    public void close() {
        //does nothing ??
    }

} //end of class
